@@ -34,6 +34,7 @@ method_all = ['1dist', 'filler', 'prop']
 method = '1dist'
 method_use_age_floor = True
 method_use_second_pass = True
+method_use_seq = True
 pop_age = 'Age'
 pop_birth = 'BTH'
 pop_at_least = 1
@@ -114,6 +115,7 @@ def is_method(x):
 
 is_bool_afloor = lambda x: is_bool(x, 'method_use.age_floor')
 is_bool_mu2p = lambda x: is_bool(x, 'method_use.second_pass')
+is_bool_seq = lambda x: is_bool(x, 'method_use.seq')
 is_containing_comp_end = lambda x: is_list_containing(x, 'comp.end', False)
 is_containing_netc = lambda x: is_list_containing(x, 'comp_neg.etc', False)
 is_containing_petc = lambda x: is_list_containing(x, 'comp_pos.etc', False)
@@ -177,7 +179,8 @@ _global_config = {
     'method': method,
     'method_use': {
         'age_floor': method_use_age_floor,
-        'second_pass': method_use_second_pass
+        'second_pass': method_use_second_pass,
+        'seq': method_use_seq
     },
     'pop': {
         'age': pop_age,
@@ -194,11 +197,14 @@ _registered_options = {
         defval = age_down_to,
         doc =\
             "\n (int)\n    " +\
-            "A limit to which neighbouring ages are considered by the " +\
-            "estime2 package. Counter-adjustments are applied " +\
-            "from the maximum neighbouring age and " +\
+            "(40 by default) A limit to which neighbouring ages are " +\
+            "considered by the estime2 package. Counter-adjustments " +\
+            "are applied from the maximum neighbouring age and " +\
             "all the way down to min(age.floor, max(-1, " +\
-            "age of modification - age.down_to)).",
+            "age of modification - `age.down_to`)) if " +\
+            "`method_use.age_floor` is True, or max(-1," +\
+            "age of modification - `age.down_to`) if " +\
+            "`method_use.age_floor` is False.",
         validator = is_int_age_down_to
     ),
     'age.floor': RegisteredOption(
@@ -206,11 +212,14 @@ _registered_options = {
         defval = age_floor,
         doc =\
             "\n (int)\n    " +\
-            "A bottom level of old neighbouring ages of the " +\
-            "problematic age. The minimum counter-adjusted " +\
-            "neighbouring age will be either age.floor or " +\
-            "max(-1, age of modification - age.down_to), whichever " +\
-            "is smaller.",
+            "(50 by default) A bottom level of old neighbouring ages " +\
+            "of the problematic age. If `method_use.age_floor` is " +\
+            "True, then the minimum counter-adjusted neighbouring age " +\
+            "will be either `age.floor` or max(-1, age of modification " +\
+            "- `age.down_to`), whichever is smaller. If " +\
+            "`method_use.age_floor` is False, then it will be " +\
+            "max(-1, age of modification - `age.down_to`) and this " +\
+            "option will be ignored.",
         validator = is_int_age_floor
     ),
     'age.max': RegisteredOption(
@@ -218,10 +227,10 @@ _registered_options = {
         defval = age_max,
         doc =\
             "\n (int)\n    " +\
-            'A maximum individual age. Any age bigger than ' +\
-            'this option is classified as the "plus category." ' +\
-            'That is, if `age.max` == 99 (default), then any age ' +\
-            'greater than or equal to 100 is classified as `100+`.', 
+            '(99 by default) A maximum individual age. Any age bigger ' +\
+            'than this option is classified as the "plus category." ' +\
+            'That is, if `age.max` == 99, then any age greater than or' +\
+            'equal to 100 is classified as `100+`.', 
         validator = is_int_age_max
     ),
     'age.prop_size': RegisteredOption(
@@ -229,9 +238,9 @@ _registered_options = {
         defval = age_prop_size,
         doc =\
             "\n (int)\n    " +\
-            'A size of period to use when applying the ' +\
-            'proportional method. It must be a positive integer.' +\
-            'Default is 5.',
+            '(5 by default) A size of period to use when applying the ' +\
+            'proportional method or the sequential one-distribution ' +\
+            '(see `method_use.seq`). It must be a positive integer.',
         validator = is_pos_age_prop_size
     ),
     'comp.end': RegisteredOption(
@@ -314,7 +323,7 @@ _registered_options = {
         defval = comp_neg_put_etc_before,
         doc =\
             "\n (bool)\n " +\
-            "A bool (`False` by default) that determines whether " +\
+            "(`False` by default) A bool that determines whether " +\
             "or not components in `comp_neg.etc` should precede " +\
             "`comp_neg.interprov` and `comp_neg.intraprov` when " +\
             "applying corrections. If `True`, the precedence of " +\
@@ -323,7 +332,7 @@ _registered_options = {
             "migrations. If `False`, it is " +\
             "set to follow interprovincial or intraprovincial " +\
             "migration, depending on the regional level of the " +\
-            "population tale.",
+            "population table.",
         validator = is_etc_n_b4_inters
     ),
     'comp_pos.temp_in': RegisteredOption(
@@ -406,7 +415,7 @@ _registered_options = {
             "migrations. If `False`, it is " +\
             "set to follow interprovincial or intraprovincial " +\
             "migration, depending on the regional level of the " +\
-            "population tale.",
+            "population table.",
         validator = is_etc_p_b4_inters
     ),
     'method': RegisteredOption(
@@ -414,9 +423,9 @@ _registered_options = {
         defval = method,
         doc =\
             "\n (str)\n    " +\
-            "A method to apply counter-adjustments to a population " +\
-            "table. Must be either one of '1dist', 'filler', " +\
-            "or 'prop'.",
+            "('1dist' by default) A method to apply " +\
+            "counter-adjustments to a population table. Must be either " +\
+            "one of '1dist', 'filler', or 'prop'.",
         validator = is_method
     ),
     'method_use.age_floor': RegisteredOption(
@@ -425,8 +434,8 @@ _registered_options = {
         doc =\
             "\n (bool)\n    " +\
             "(True by default) If True, then the estime2 package " +\
-            "uses age.floor option when calculating the minimum " +\
-            "neighbouring age for the problematic record.",
+            "uses `age.floor` option when calculating the minimum " +\
+            "neighbouring age of the problematic record.",
         validator = is_bool_afloor
     ),
     'method_use.second_pass': RegisteredOption(
@@ -443,6 +452,26 @@ _registered_options = {
             "nothing more.",
         validator = is_bool_mu2p
     ),
+    'method_use.seq': RegisteredOption(
+        key = 'method_use.seq',
+        defval = method_use_seq,
+        doc =\
+            "(True by default) If True, then the one-distribution " +\
+            "method will distribute the value of 1's to neighbouring " +\
+            'ages and use the second pass (if applicable) ' +\
+            '"sequentially". That is, when neighbouring ages are ' +\
+            "given, then the process will look at the first " +\
+            "`age.prop_size`-years of records and apply the " +\
+            "one-distribution method and the second pass " +\
+            "(if applicable) within those years. Then, it moves on " +\
+            "to the next `age.prop_size`-years of records, does the " +\
+            "same job, and continues until the minimum neighbouring " +\
+            "age is met. If this option is False, then the " +\
+            'one-distribution is applied "all at once" from the ' +\
+            'maximum to the minimum neighbouring age, and the second ' +\
+            'pass as well (if applicable).',
+        validator = is_bool_seq
+    ),
     'pop.age': RegisteredOption(
         key = "pop.age",
         defval = pop_age,
