@@ -1121,12 +1121,12 @@ class ProvPopTable(pd.DataFrame):
         I = self.get_I()
         I = I.query('I != 0').sort_values(pop_groups)
         problematic = I.iloc[0, :] # choose the youngest
-        problematic_sex = problematic[self.__pop_sex]
-        problematic_age = problematic[self.__pop_age]
+        problematic_sex = problematic[pop_groups[0]]
+        problematic_age = problematic[pop_groups[1]]
         problematic_val = problematic['I']
         calculated_pop = self.calculate_pop()
         comp_in_neg = comp in comp_neg # comp is in comp_pos otherwise
-        comp_in_comp_end = comp in self.__comp_end
+        comp_in_comp_end = comp in self.get_comp_end()
         age_is_max = problematic_age.is_max()
 
         # Get the corresponding age(s) of comp to modify & counter-adjust
@@ -1143,16 +1143,6 @@ class ProvPopTable(pd.DataFrame):
         correctable_in_comp = self\
             .loc[lambda df: df[self.__pop_sex] == problematic_sex]\
             [cols_required]
-        to_modify_records = calculate_modifiable_in_comp(
-            df_pop_end = correctable_in_pop_end,
-            df_comp = correctable_in_comp,
-            pop_age = self.__pop_age,
-            pop_end = self.__pop_end,
-            comp = comp,
-            problematic_age = problematic_age,
-            to_modify_age = to_modify_age,
-            comp_in_neg = comp_in_neg
-        )
         to_counter_records = calculate_counter_adjustable_in_comp(
             df_pop_end = correctable_in_pop_end,
             df_comp = correctable_in_comp,
@@ -1163,6 +1153,19 @@ class ProvPopTable(pd.DataFrame):
             comp_in_comp_end = comp_in_comp_end,
             comp_in_neg = comp_in_neg
         )
+        sum_counter_adjustable =\
+            to_counter_records.iloc[:, -1].values.sum()
+        to_modify_records = calculate_modifiable_in_comp(
+            df_pop_end = correctable_in_pop_end,
+            df_comp = correctable_in_comp,
+            pop_age = self.__pop_age,
+            pop_end = self.__pop_end,
+            comp = comp,
+            problematic_age = problematic_age,
+            to_modify_age = to_modify_age,
+            comp_in_neg = comp_in_neg,
+            sum_counter_adjustable = sum_counter_adjustable
+        )
         result = {
             'records.to_modify': to_modify_records,
             'records.to_counter': to_counter_records
@@ -1259,7 +1262,7 @@ class ProvPopTable(pd.DataFrame):
         
         comp_in_comp_end = comp in self.__comp_end
         pop_groups = [self.__pop_sex, self.__pop_age]
-        comp_J = "{0}_J".format(comp)
+        comp_J = f"{comp}_J"
         dfs_comp = None
         if J is None:
             dfs_comp = self.get_J(comp) # sanity check of comp done here
@@ -1271,28 +1274,48 @@ class ProvPopTable(pd.DataFrame):
             .sort_values(pop_groups, ascending = [True, False])
         to_modify_val_total = df_comp_modifiable[comp_J].sum()
 
-        comp_K = "{0}_K".format(comp)
+        comp_K = f"{comp}_K"
         result = []
         max_correctables =\
             df_comp_counter_adjust_reversed.iloc[:, -1].values.copy()
 
         if method == '1dist':
-            for index, row in df_comp_counter_adjust_reversed.iterrows():
-                if row[comp_J] > 0 and to_modify_val_total != 0:
-                    result.append(1)
-                    to_modify_val_total -= 1
-                else:
-                    result.append(0)
+            apply_sequentially = get_option('method_use.seq')
             use_2nd_pass = get_option('method_use.second_pass')
-            if to_modify_val_total != 0 and use_2nd_pass:
-                # The first pass wasn't enough so that to_modify_val_total
-                # didn't come down to 0.
-                # Apply the second pass only if the option says so.
-                max_correctables -= np.array(result)
-                for i, item in enumerate(max_correctables):
-                    if item > 0 and to_modify_val_total != 0:
-                        result[i] += 1
+            if apply_sequentially:
+                seq_size = get_option('age.prop_size')
+                len_mc = len(max_correctables)
+                for i in np.arange(len_mc, step = seq_size):
+                    period = [i, min(i + seq_size, len_mc)]
+                    mc_seq = max_correctables[period[0]:period[1]]
+                    for num in mc_seq:
+                        if num > 0 and to_modify_val_total != 0:
+                            result.append(1)
+                            to_modify_val_total -= 1
+                        else:
+                            result.append(0)
+                    if to_modify_val_total != 0 and use_2nd_pass:
+                        mc_seq -= np.array(result[period[0]:period[1]])
+                        for j, num2 in enumerate(mc_seq):
+                            if num2 > 0 and to_modify_val_total != 0:
+                                result[j + i] += 1
+                                to_modify_val_total -= 1
+            else:
+                for ind, row in df_comp_counter_adjust_reversed.iterrows():
+                    if row[comp_J] > 0 and to_modify_val_total != 0:
+                        result.append(1)
                         to_modify_val_total -= 1
+                    else:
+                        result.append(0)
+                if to_modify_val_total != 0 and use_2nd_pass:
+                    # The first pass wasn't enough so that 
+                    # to_modify_val_total didn't come down to 0.
+                    # Apply the second pass only if the option says so.
+                    max_correctables -= np.array(result)
+                    for i, item in enumerate(max_correctables):
+                        if item > 0 and to_modify_val_total != 0:
+                            result[i] += 1
+                            to_modify_val_total -= 1
         elif method == 'filler':
             for index, row in df_comp_counter_adjust_reversed.iterrows():
                 if row[comp_J] > 0 and to_modify_val_total != 0:
@@ -1599,35 +1622,48 @@ def calculate_counter_adjustable_in_comp(
         # This will never happen since to_counter_age_min is always 
         # at least -1.
         raise NotImplementedError
-    for pop_end_val, comp_val in zip(pop_end_values, comp_values):
-        if pop_end_val <= 0:
+    zippc = zip(pop_end_values, comp_values)
+    for pop_end_minus_at_least_val, comp_val in zippc:
+        if pop_end_minus_at_least_val <= 0:
             # If "end-of-period pop - 1" <= 0 (where 1 == pop.at_least),
             # then that neighbouring age record can't absorb any correction
             to_append = 0
         elif comp_in_neg:
             # If "end-of-period pop - 1" > 0, i.e. it is correctable,
             # then see if the component is negative.
-            # If it is, then the maximum correction (i.e. increase in the
-            # component value of the record, like increasing the death from
-            # 2 to 3) of this record must be the "end-of-period pop of that 
-            # record - 1" because if the negative component increases more
-            # than that, then the "end-of-period population - 1" of the 
-            # neighbouring age record will go below 1.
-            to_append = pop_end_val
+            # If it is, then the maximum correction 
+            # (i.e. increase in the component value of the record, like 
+            # increasing the death from 2 to 3) of this record must 
+            # be the "end-of-period pop of that record - 1" REGARDLESS of
+            # the value at the problematic component. 
+            # This is because:
+            #     1. if the negative component increases more than that, 
+            #         then the "end-of-period population - 1" of the 
+            #         neighbouring age record will go below 1.
+            #     2. all the values in the population table is (supposedly)
+            #         nonnegative, and a counter-adjustment to negative 
+            #         problematic components will increase the component 
+            #         value at that record (e.g. death value 2 -> 3). There
+            #         is no limit as to how much it can increase so long as
+            #         the "end-of-period pop of that record - 1" remains
+            #         nonnegative.
+            to_append = pop_end_minus_at_least_val
         else:
             # If "end-of-period pop - 1" > 0, i.e. it is correctable,
-            # and the component is positive, then the maximum correction 
+            # then see if the component is positive.
+            # If it is, then the maximum correction 
             # (i.e. "decrease" in the component value of the record, like
             # decreasing the immigration from 5 to 4) of this record must
             # be the minimum of "end-of-period pop - 1" and the component 
-            # value. This is because, in the positive component, the 
+            # value. 
+            # This is because, in the positive component, the 
             # component value has to decrease (not increase) in 
             # neighbouring ages in order to balance out the
             # modification at the problematic record. If the 
             # counter-adjustment goes below this minimum value, then either
-            # the end-of-period population or the component value will 
-            # become negative.
-            to_append = min(pop_end_val, comp_values)
+            # the "end-of-period population - 1" or the component value 
+            # will become negative.
+            to_append = min(pop_end_minus_at_least_val, comp_val)
         counter_adjustable.append(to_append)
 
     df_comp_counter_adjust[f"{comp}_J"] = counter_adjustable
@@ -1643,13 +1679,15 @@ def calculate_modifiable_in_comp(
     comp: str,
     problematic_age: Age,
     to_modify_age: Union[int, List[int]],
-    comp_in_neg: bool
+    comp_in_neg: bool,
+    sum_counter_adjustable: int
 ):
     '''
     Compute the maximum modifiable value of `comp` in `df_comp` based on 
     `pop_end` in `df_pop_end`, the information regarding the 
-    `problematic_age`, `to_modify_age`, and `comp_in_neg`. `pop_age` is the
-    name of the age column for both `df_pop_end` and `df_comp`.
+    `problematic_age`, `to_modify_age`, `comp_in_neg`, and 
+    `sum_counter_adjustable`. `pop_age` is the name of the age column for
+    both `df_pop_end` and `df_comp`.
     '''
 
     at_least = get_option('pop.at_least')
@@ -1677,40 +1715,40 @@ def calculate_modifiable_in_comp(
     if isinstance(to_modify_age, int):
         modifiable_val_in_comp = df_comp_modifiable[comp].values[0]
         if comp_in_neg:
-            # `modifiable` also has to consider the sum of the
-            # counter-adjustable values in `min`, but it is not done here
-            # since it is deemed to be always greater than these two 
-            # numbers in the following min function.
             modifiable = min(
                 abs_modifiable_val_in_pop_end,
-                modifiable_val_in_comp
+                modifiable_val_in_comp,
+                sum_counter_adjustable
             )
         else:
-            modifiable = abs_modifiable_val_in_pop_end
+            modifiable = min(
+                abs_modifiable_val_in_pop_end,
+                sum_counter_adjustable
+            )
     elif isinstance(to_modify_age, list):
         modifiable_val_in_comp = df_comp_modifiable[comp]
-        modifiable_val_in_comp_0 = modifiable_val_in_comp.values[0]
-        modifiable_val_in_comp_1 = modifiable_val_in_comp.values[1]
         modifiable_0 = None
         modifiable_1 = None
         if comp_in_neg:
+            modifiable_val_in_comp_0 = modifiable_val_in_comp.values[0]
+            modifiable_val_in_comp_1 = modifiable_val_in_comp.values[1]
             modifiable_1 = min(
-                abs_modifiable_val_in_pop_end, 
-                modifiable_val_in_comp_1
+                abs_modifiable_val_in_pop_end,
+                modifiable_val_in_comp_1,
+                sum_counter_adjustable
+            )
+            modifiable_0 = min(
+                abs_modifiable_val_in_pop_end - modifiable_1, 
+                modifiable_val_in_comp_0,
+                sum_counter_adjustable - modifiable_1
             )
-            modifiable_0 = None
-            leftover = None
-            if modifiable_1 == abs_modifiable_val_in_pop_end:
-                modifiable_0 = 0
-            else:
-                leftover = abs_modifiable_val_in_pop_end -\
-                    modifiable_val_in_comp_1
-                modifiable_0 = min(leftover, modifiable_val_in_comp_0)
         else:
-            modifiable_0 =\
-                (abs_modifiable_val_in_pop_end // 2)
-            modifiable_1 =\
-                abs_modifiable_val_in_pop_end - modifiable_0
+            to_divide = min(
+                abs_modifiable_val_in_pop_end,
+                sum_counter_adjustable
+            )
+            modifiable_0 = to_divide // 2
+            modifiable_1 = to_divide - modifiable_0
         modifiable = [modifiable_0, modifiable_1]
     else:
         raise NotImplementedError
