@@ -26,7 +26,7 @@ RegionSAD = ['RegionCode', 'Sex', 'Age', 'RefDate']
 RegionSA = RegionSAD[0:3]
 RegionSD = list(np.array(RegionSAD)[[0,1,3]])
 RegionS = RegionSA[0:2]
-    
+
 def arrange_by_lvl(df, levels, value_name):
     age_cols = [col for col in df if col.startswith('A_')]
     result =\
@@ -43,8 +43,9 @@ def arrange_by_lvl(df, levels, value_name):
         )
     return result
 
-def call_tbl():
-    tbl_name = [x for x in os.listdir('..') if x[0] == 'G'][0]
+def call_tbl(use_copy: bool = False):
+    listed_files = [x for x in os.listdir('..') if x[0] == 'G']
+    tbl_name = listed_files[0] if use_copy else listed_files[1]
     tbl = pd.read_excel('../' + tbl_name, sheet_name = 1)
     poptbl = ProvPopTable(tbl)
     return poptbl
@@ -600,33 +601,54 @@ def run4():
         df_comp_counter_adjust_reversed.iloc[:, -1].values.copy()
 
     if method == '1dist':
-        for index, row in df_comp_counter_adjust_reversed.iterrows():
-            if row[comp_J] > 0 and to_modify_val_total != 0:
-                result.append(1)
-                to_modify_val_total -= 1
-            else:
-                result.append(0)
-        print("result so far:")
-        print(result)
+        apply_sequentially = get_option('method_use.seq')
         use_2nd_pass = get_option('method_use.second_pass')
-        if to_modify_val_total != 0 and use_2nd_pass:
-            # The first pass wasn't enough so that to_modify_val_total
-            # didn't come down to 0.
-            # Apply the second pass only if the option says so
-            print("The second pass has started.")
-            max_correctables -= np.array(result)
-            print("df_comp_counter_adjust_reversed:")
-            print(df_comp_counter_adjust_reversed)
-            print("New max_correctables:")
-            print(max_correctables)
-            for i, item in enumerate(max_correctables):
-                if item > 0 and to_modify_val_total != 0:
-                    result[i] += 1
+        if apply_sequentially:
+            seq_size = get_option('age.prop_size')
+            len_mc = len(max_correctables)
+            for i in np.arange(len_mc, step = seq_size):
+                period = [i, min(i + seq_size, len_mc)]
+                mc_seq = max_correctables[period[0]:period[1]]
+                print(mc_seq)
+                for num in mc_seq:
+                    if num > 0 and to_modify_val_total != 0:
+                        result.append(1)
+                        to_modify_val_total -= 1
+                    else:
+                        result.append(0)
+                if to_modify_val_total != 0 and use_2nd_pass:
+                    mc_seq -= np.array(result[period[0]:period[1]])
+                    for j, num2 in enumerate(mc_seq):
+                        if num2 > 0 and to_modify_val_total != 0:
+                            result[j + i] += 1
+                            to_modify_val_total -= 1
+        else:
+            for index, row in df_comp_counter_adjust_reversed.iterrows():
+                if row[comp_J] > 0 and to_modify_val_total != 0:
+                    result.append(1)
                     to_modify_val_total -= 1
-                print("to_modify_val_total:")
-                print(to_modify_val_total)
-                print("result:")
-                print(result)
+                else:
+                    result.append(0)
+            print("result so far:")
+            print(result)
+            if to_modify_val_total != 0 and use_2nd_pass:
+                # The first pass wasn't enough so that to_modify_val_total
+                # didn't come down to 0.
+                # Apply the second pass only if the option says so
+                print("The second pass has started.")
+                max_correctables -= np.array(result)
+                print("df_comp_counter_adjust_reversed:")
+                print(df_comp_counter_adjust_reversed)
+                print("New max_correctables:")
+                print(max_correctables)
+                for i, item in enumerate(max_correctables):
+                    if item > 0 and to_modify_val_total != 0:
+                        result[i] += 1
+                        to_modify_val_total -= 1
+                    print("to_modify_val_total:")
+                    print(to_modify_val_total)
+                    print("result:")
+                    print(result)
 
     elif method == 'filler':
         for index, row in df_comp_counter_adjust_reversed.iterrows():
@@ -903,3 +925,231 @@ def run6():
     print('')
     print(poptbl_1010_fixed_prop.summary())
     print('')
+
+def run7(comp = 'DTH'):
+
+    self = call_tbl()
+        
+    # Get basic options
+    comp_neg = self.get_comp_neg()
+    comp_pos = self.get_comp_pos()
+    comps = comp_neg + comp_pos
+    # raise_if_not_subset([comp], comps, 'comp', 'components of self')
+    at_least = get_option('pop.at_least')
+    pop_groups = self.get_pop_groups()
+    cols_required = pop_groups.copy()
+    cols_required.append(comp)    
+
+    # Get properties of the youngest problematic record
+    ## .sort_values() will be deprecated in later versions of pandas
+    I = self.get_I()
+    I = I.query('I != 0').sort_values(pop_groups)
+    problematic = I.iloc[0, :] # choose the youngest
+    problematic_sex = problematic[pop_groups[0]]
+    problematic_age = problematic[pop_groups[1]]
+    problematic_val = problematic['I']
+    calculated_pop = self.calculate_pop()
+    comp_in_neg = comp in comp_neg # comp is in comp_pos otherwise
+    comp_in_comp_end = comp in self.get_comp_end()
+    age_is_max = problematic_age.is_max()
+
+    # Get the corresponding age(s) of comp to modify & counter-adjust
+    ages = calculate_ages_to_modify_and_counter(
+        problematic_age, 
+        comp_in_comp_end
+    )
+    to_modify_age = ages['age.to_modify']
+    to_counter_age = ages['age.to_counter']
+
+    # Get the maximum amounts modifiable & counter-adjustable
+    correctable_in_pop_end = calculated_pop.copy()\
+        .loc[lambda df: df[pop_groups[0]] == problematic_sex]
+    correctable_in_comp = self\
+        .loc[lambda df: df[pop_groups[0]] == problematic_sex]\
+        [cols_required]
+
+    # calculate_counter_adjustable_in_comp(...)
+    df_pop_end = correctable_in_pop_end
+    df_comp = correctable_in_comp
+    pop_age = self.get_pop_groups()[1]
+    pop_end = self.get_pop_end()
+    to_counter_age_min = to_counter_age[0]
+    to_counter_age_max = to_counter_age[1]
+    df_pop_end_compare = None
+    if comp_in_comp_end:
+        df_pop_end_compare = df_pop_end\
+            .loc[lambda df: df[pop_age] >= to_counter_age_min]\
+            .loc[lambda df: df[pop_age] <= to_counter_age_max]
+    else:
+        df_pop_end_compare = df_pop_end\
+            .loc[lambda df: df[pop_age] >= to_counter_age_min + 1]\
+            .loc[lambda df: df[pop_age] <= to_counter_age_max + 1]
+    df_pop_end_compare[pop_end] -= at_least
+    df_comp_counter_adjust = df_comp\
+        .loc[lambda df: df[pop_age] >= to_counter_age_min]\
+        .loc[lambda df: df[pop_age] <= to_counter_age_max]
+
+    pop_end_values = df_pop_end_compare[pop_end].values
+    comp_values = df_comp_counter_adjust[comp].values
+    counter_adjustable = []
+    to_append = None
+    if len(comp_values) != len(pop_end_values):
+        # This will never happen since to_counter_age_min is always 
+        # at least -1.
+        raise NotImplementedError
+
+    zippc = zip(pop_end_values, comp_values)
+    for pop_end_minus_at_least_val, comp_val in zippc:
+        if pop_end_minus_at_least_val <= 0:
+            to_append = 0
+        elif comp_in_neg:
+            to_append = pop_end_minus_at_least_val
+        else:
+            to_append = min(pop_end_minus_at_least_val, comp_val)
+        counter_adjustable.append(to_append)
+
+    df_comp_counter_adjust[f"{comp}_J"] = counter_adjustable
+    del df_comp_counter_adjust[comp]
+
+    return df_comp_counter_adjust
+
+def run8(comp = 'DTH', use_copy = True):
+
+    self = call_tbl(use_copy)
+    
+    # Get basic options
+    comp_neg = self.get_comp_neg()
+    comp_pos = self.get_comp_pos()
+    comps = comp_neg + comp_pos
+    # raise_if_not_subset([comp], comps, 'comp', 'components of self')
+    at_least = get_option('pop.at_least')
+    pop_groups = self.get_pop_groups()
+    cols_required = pop_groups.copy()
+    cols_required.append(comp)    
+
+    # Get properties of the youngest problematic record
+    ## .sort_values() will be deprecated in later versions of pandas
+    I = self.get_I()
+    I = I.query('I != 0').sort_values(pop_groups)
+    problematic = I.iloc[0, :] # choose the youngest
+    problematic_sex = problematic[pop_groups[0]]
+    problematic_age = problematic[pop_groups[1]]
+    problematic_val = problematic['I']
+    calculated_pop = self.calculate_pop()
+    comp_in_neg = comp in comp_neg # comp is in comp_pos otherwise
+    comp_in_comp_end = comp in self.get_comp_end()
+    age_is_max = problematic_age.is_max()
+
+    # Get the corresponding age(s) of comp to modify & counter-adjust
+    ages = calculate_ages_to_modify_and_counter(
+        problematic_age, 
+        comp_in_comp_end
+    )
+    to_modify_age = ages['age.to_modify']
+    to_counter_age = ages['age.to_counter']
+
+    # Get the maximum amounts modifiable & counter-adjustable
+    correctable_in_pop_end = calculated_pop.copy()\
+        .loc[lambda df: df[pop_groups[0]] == problematic_sex]
+    correctable_in_comp = self\
+        .loc[lambda df: df[pop_groups[0]] == problematic_sex]\
+        [cols_required]
+    
+    to_counter_records = run7(comp)
+    sum_counter_adjustable =\
+        to_counter_records.iloc[:, -1].values.sum()    
+    df_pop_end = correctable_in_pop_end
+    df_comp = correctable_in_comp
+    pop_age = self.get_pop_groups()[1]
+    pop_end = self.get_pop_end()
+    # comp = comp,
+    # problematic_age = problematic_age,
+    # to_modify_age = to_modify_age,
+    # comp_in_neg = comp_in_neg,
+    # sum_counter_adjustable = sum_counter_adjustable
+
+    # calculate_modifiable_in_comp(...)
+    pop_end_query = None
+    if age_is_max:
+        pop_end_query = f"{pop_age} > {get_option('age.max')}"
+    else:
+        pop_end_query = f"{pop_age} == {problematic_age}"
+    df_pop_end_problematic = df_pop_end.query(pop_end_query)
+    df_comp_modifiable = None
+    if isinstance(to_modify_age, int):
+        df_comp_modifiable = df_comp\
+            .loc[lambda df: df[pop_age] == to_modify_age]
+    elif isinstance(to_modify_age, list):
+        df_comp_modifiable = df_comp\
+            .loc[lambda df: df[pop_age] <= to_modify_age[1]]\
+            .loc[lambda df: df[pop_age] >= to_modify_age[0]]
+    else:
+        raise NotImplementedError
+
+    print("df_pop_end_problematic:")
+    print(df_pop_end_problematic)
+    print("")
+    print("df_comp_modifiable:")
+    print(df_comp_modifiable)
+    print("")
+
+    modifiable_val_in_pop_end = df_pop_end_problematic[pop_end].values[0]
+    abs_modifiable_val_in_pop_end = abs(modifiable_val_in_pop_end)
+    print("abs_modifiable_val_in_pop_end:", abs_modifiable_val_in_pop_end)
+    modifiable = None
+    if isinstance(to_modify_age, int):
+        modifiable_val_in_comp = df_comp_modifiable[comp].values[0]
+        print("modifiable_val_in_comp:", modifiable_val_in_comp)
+        print("sum_counter_adjustable:", sum_counter_adjustable)
+        print("")
+        if comp_in_neg:
+            modifiable = min(
+                abs_modifiable_val_in_pop_end,
+                modifiable_val_in_comp,
+                sum_counter_adjustable
+            )
+        else:
+            modifiable = min(
+                abs_modifiable_val_in_pop_end,
+                sum_counter_adjustable
+            )
+        print("modifiable:", modifiable)
+        print("")
+    elif isinstance(to_modify_age, list):
+        modifiable_val_in_comp = df_comp_modifiable[comp]
+        modifiable_0 = None
+        modifiable_1 = None
+        print("modifiable_val_in_comp:")
+        print(modifiable_val_in_comp)
+        print("sum_counter_adjustable:", sum_counter_adjustable)
+
+        if comp_in_neg:
+            modifiable_val_in_comp_0 = modifiable_val_in_comp.values[0]
+            modifiable_val_in_comp_1 = modifiable_val_in_comp.values[1]
+            print("modifiable_val_in_comp_0:", modifiable_val_in_comp_0)
+            print("modifiable_val_in_comp_1:", modifiable_val_in_comp_1)
+            print("")
+            modifiable_1 = min(
+                abs_modifiable_val_in_pop_end,
+                modifiable_val_in_comp_1,
+                sum_counter_adjustable
+            )
+            modifiable_0 = min(
+                abs_modifiable_val_in_pop_end - modifiable_1, 
+                modifiable_val_in_comp_0,
+                sum_counter_adjustable - modifiable_1
+            )
+            print("modifiable_0:", modifiable_0)
+            print("modifiable_1:", modifiable_1)
+        else:
+            to_divide = min(
+                abs_modifiable_val_in_pop_end,
+                sum_counter_adjustable
+            )
+            modifiable_0 = to_divide // 2
+            modifiable_1 = to_divide - modifiable_0
+            print("to_divide:", to_divide)
+            print("modifiable_0:", modifiable_0)
+            print("modifiable_1:", modifiable_1)
+    else:
+        raise NotImplementedError
